#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "tcl.h"
#include "msql.h"

static int debug = 0;
struct ClientData 
{
  m_result *resarray[256];
  int midx;
};


/* static int Dragdrop_Drag(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]); */

static int msqlConnect_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlSelectDB_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlQuery_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlFreeResult_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlStoreResult_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlFetchRow_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlDataSeek_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlDataSeek_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlNumRows_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlFetchField_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlFieldSeek_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlNumFields_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlClose_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlListDBs_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlListTables_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlListFields_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);
static int msqlListIndex_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[]);



static int msqlConnect_Cmd(ClientData clientData, Tcl_Interp *interp, 
                           int argc, char *argv[])
{
  int i;
  char *host = NULL, buf[20]; /* needs to hold one integer */
  

#ifdef DEBUG
  fprintf(stderr, "msqlConnect, args = %d, val1 = %s, val2 = %s\n", argc,
	  argv[0], argv[1]);
#endif
  
  if (argc == 2) {
    host = argv[1];
  }
  i = msqlConnect(host);
  if (i < 0) {
    interp->result = "Cannot connect to remote mSQL host";
    return TCL_ERROR;
  }
  sprintf(buf, "%d", i);
  interp->result = strdup(buf);
  
  return TCL_OK;
}



static int msqlSelectDB_Cmd(ClientData clientData, Tcl_Interp *interp, 
			    int argc, char *argv[])
{   
  int sock, r;
  
  if (argc != 3) {
    interp->result = "msqlSelectDB needs two params";
    return TCL_ERROR;
  }
  sock = atoi(argv[1]);
  r = msqlSelectDB(sock, argv[2]);
  
  if (r < 0) {
      interp->result = msqlErrMsg;
      return TCL_ERROR;
  }
  interp->result = "";
  return TCL_OK;
}

static int msqlQuery_Cmd(ClientData clientData, Tcl_Interp *interp, 
			 int argc, char *argv[])
{   
  int sock, r;
  char buf[20];
  
  
  if (argc != 3) {
    interp->result = "msqlQuery needs two params";
    return TCL_ERROR;
  }
  sock = atoi(argv[1]);
  r = msqlQuery(sock, argv[2]);
  
  if (r < 0) {
      interp->result = msqlErrMsg;
      return TCL_ERROR;
  }

  sprintf(buf, "%d", r);
  interp->result = strdup(buf);

  return TCL_OK;
}

static int msqlFreeResult_Cmd(ClientData clientData, Tcl_Interp *interp, 
			      int argc, char *argv[])
{   
  int sock, r;
  char buf[20];
  m_result        *res;
  struct ClientData *cd = (struct ClientData *) clientData;
  
  if (argc != 2) {
    interp->result = "msqlFreeResult needs one params";
    return TCL_ERROR;
  }
  r = atoi(argv[1]);
  msqlFreeResult(cd->resarray[r]);
  if (cd->midx == r) {
    cd->midx--;
  }

  return TCL_OK;
}

static int msqlStoreResult_Cmd(ClientData clientData, Tcl_Interp *interp, 
			       int argc, char *argv[])
{   
  m_result        *res;
  char buf[20];
  struct ClientData *cd = (struct ClientData *) clientData;

  res = msqlStoreResult();
  
  if (res == NULL) {
      interp->result = msqlErrMsg;
      return TCL_ERROR;
  }
  
  cd->resarray[cd->midx] = res;
  sprintf(buf, "%d", cd->midx);
  cd->midx++;
  interp->result = strdup(buf);
  
  return TCL_OK;
}

static int msqlFetchRow_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  m_result        *res;
  m_row           row;
  int r, size, i;
  char *cp;
  struct ClientData *cd = (struct ClientData *) clientData;

  if (argc != 2) {
    interp->result = "msqlFetchRow needs one params";
    return TCL_ERROR;
  }

  r = atoi(argv[1]);
  res = cd->resarray[r];
  row = msqlFetchRow(res);
  if (row == NULL) {
    interp->result = "";
    return TCL_OK;
  }

  /* now create a tcl list and return result */
  
  for (size = 0, i = 0; i < msqlNumFields(res); i++) {
    size += (3 + strlen(row[i]));
  }
  cp = (char *) malloc((size + 1) * sizeof(char));
  if (cp == NULL) {
    exit(5);
  }
  cp[0] = '\0';
  for (i = 0; i < msqlNumFields(res); i++) {
    strcat(cp, "{");
    strcat(cp, row[i]);
    strcat(cp, "} ");
  }

  interp->result = cp;
  return TCL_OK;
}

static int msqlDataSeek_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;
}

static int msqlNumRows_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;
}

static int msqlFetchField_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;
}

static int msqlFieldSeek_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;
}

static int msqlNumFields_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;
}

static int msqlClose_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  int sock, r;
  
  if (argc != 2) {
    interp->result = "msqlClose needs one param";
    return TCL_ERROR;
  }
  sock = atoi(argv[1]);
  msqlClose(sock);
  interp->result = "";
  return TCL_OK;
}

static int msqlListDBs_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;
}

static int msqlListTables_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;

}

static int msqlListFields_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;

}

static int msqlListIndex_Cmd(ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{   
  interp->result = "not implemented";
  return TCL_ERROR;
}


/* ---------------------------------------------------------------------- */

int msql_Init(Tcl_Interp *interp) {
  static char initCmd[] = "";
  int i;
  char *lib;
  Tcl_DString path;
  static struct ClientData c;

  c.midx = 0;


  /* Tcl_SetVar(interp, DVERSN, VERSION, TCL_GLOBAL_ONLY);
     Tcl_SetVar(interp, DPATCH, VERSION PATCHLEVEL, TCL_GLOBAL_ONLY);

     Tcl_DStringInit(&path);
     Tcl_DStringAppend(&path,
		    Tcl_GetVar(interp, "auto_path", TCL_GLOBAL_ONLY),
		    ALL);
     lib = Tcl_GetVar2(interp, "env", DRAGDROPLIBVAR, TCL_GLOBAL_ONLY);
  
  if (NMT(lib)) {
    Tcl_DStringAppendElement(&path, lib);
    Tcl_SetVar(interp, DLIBRY, lib, TCL_GLOBAL_ONLY);
  } else {
    Tcl_SetVar(interp, DLIBRY, LIBDIR, TCL_GLOBAL_ONLY);
    Tcl_DStringAppendElement(&path, LIBDIR);
  }
  Tcl_SetVar(interp, "auto_path", V(&path), TCL_GLOBAL_ONLY);
  Tcl_DStringFree(&path);

  c = (ClientData) DragdropAllocDataTable(interp);
  */

  Tcl_CreateCommand(interp, "msqlConnect", msqlConnect_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlSelectDB", msqlSelectDB_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlQuery", msqlQuery_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlStoreResult", msqlStoreResult_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlFreeResult", msqlFreeResult_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlFetchRow", msqlFetchRow_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlDataSeek", msqlDataSeek_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlNumRows", msqlNumRows_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlFetchField", msqlFetchField_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlFieldSeek", msqlFieldSeek_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlNumFields", msqlNumFields_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlClose", msqlClose_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlListDBs", msqlListDBs_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlListTables", msqlListTables_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlListFields", msqlListFields_Cmd, &c, NULL);
  Tcl_CreateCommand(interp, "msqlListIndex", msqlListIndex_Cmd, &c, NULL);

  /* Tcl_CreateCommand(interp, MAINCMD, Dragdrop_Cmd, c, dddel); */ 
  return Tcl_Eval(interp, initCmd);
}
